//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                    Tic-Tac-Toe
//              (C) Denis Lukianov 2002
// Example source code using C++, public domain.
//                denis@voxelsoft.com
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Header files
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <windows.h>
#include <iostream>
#include <stdlib.h>
#include <iomanip.h>
#include <time.h>
#include <conio.h>

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Classes
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Main game class
class CGame; // The game state and functionality

// Members of game class
class CPlayer; // Holds data about a player
class CBoard; // Contains an interface to access the game board
class C_AI; // Contains an interface to get computer generated moves

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Constant definitions
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Game specs
#define BOARD_WIDTH				3 // Number of squares horizontally or vertically on the square game board
#define BOARD_SIZE				9 // Number of squares on the game board, (BOARD_WIDTH * BOARD_WIDTH)
#define NUM_PLAYERS				2 // Number of players in the game, 2 for tic-tac-toe
#define MAX_LINES				8 // Number of lines possible on the game board
#define COMPUTER_DELAY_TIME		500 // To make it look like computer is thinking

// Game states
#define GAME_DRAW				(char)250 // A game was drawn, no winner

// Player types
#define PLAYER_HUMAN			0 // Human player, requires input
#define PLAYER_PC				1 // Computer player, requires AI processing

// Artificial Intelligence constants
#define AI_LINESCORE_NO					0
#define AI_LINESCORE_BATTLE				1
#define AI_LINESCORE_INTRODUCE			2
#define AI_LINESCORE_PREVENT_DEVELOP	20
#define AI_LINESCORE_DEVELOP			150
#define AI_LINESCORE_PREVENT_LOSE		1000
#define AI_LINESCORE_WIN				6000

// Technical limits
#define STRING_MARGIN			1 // To allow for safe string termination
#define MAX_PLAYER_NAME			8 // Limit the length of a player's name

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Globals - representation of the lines on a board
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

	char gcBoardLines[MAX_LINES][BOARD_WIDTH] =
	{
		// Horizontal lines
		{0, 1, 2},
		{3, 4, 5},
		{6, 7, 8},

		// Vertical lines
		{0, 3, 6},
		{1, 4, 7},
		{2, 5, 8},

		// Diagonal lines
		{0, 4, 8},
		{6, 4, 2}
	};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Function declarations
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Application entry point
int main(void);

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This class keeps player states [human/computer] and potentially the player's name
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CPlayer
{
private:

	// Human, computer, etc.. see player type definitions
	char cPlayerType;

	// This player's symbol, eg 'X' or 'O'
	char cPlayerSymbol;

	// Player's name string, zero terminated
	char cPlayerName[MAX_PLAYER_NAME + STRING_MARGIN];

public:

	// Change/retrieve player type
	void vSetPlayerType(char lcNewPlayerType);
	char cGetPlayerType(void);

	// Change/retrieve player symbol
	void vSetPlayerSymbol(char lcNewPlayerSymbol);
	char cGetPlayerSymbol(void);

	// Change/clear/retrieve player name
	bool bSetPlayerName(char *lcNewPlayerName);
	void vClearPlayerName(void);
	char *cGetPlayerName(void);

	CPlayer()
	{
		// Human player by default
		vSetPlayerType(PLAYER_HUMAN);
		
		// Blank player name
		vClearPlayerName();
	}
	~CPlayer()
	{
		// Nothing to do in destructor
	}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This class holds and allows access to the game's board
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CBoard
{
private:

	// The array representing the game board
	char cBoard[BOARD_SIZE];
	
	// Check three squares to see if they match, and if they do, return the common Symbol
	char cCheckMatch(char lcFirstSquare, char lcSecondSquare, char lcThirdSquare);

	// Work out the array square based on column and row
	char cArraySquare(char lcColumn, char lcRow);

public:

	// Clear the board
	void vClearBoard(void);
	
	// Check the square for validity
	bool bIsValidSquare(char lcSquare);

	// Interface to game board's individual squares
	char cGetSquareSymbol(char lcSquare);
	bool vSetSquareSymbol(char lcSquare, char lcSymbol);
	
	// Find if there is a completed line, and return the common Symbol
	char cGetCompletedLine(void);

	// Are all the squares taken? Are any?
	bool bIsFull(void);
	bool bIsEmpty(void);
	
	// Display board to user
	void vDrawBoardWithSquareNumbers(void);
	void vDrawBoard(void);
	
	CBoard()
	{
		// Clear board on creation
		vClearBoard();
	}
	~CBoard()
	{
		// Nothing on destruction
	}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Brains of the computer - impossible to win against
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class C_AI
{
private:

	// Does the piece belong to an enemy? or us? or is it empty?
	bool bIsEnemy(char lcFriendly, char lcSymbol);
	bool bIsFriend(char lcFriendly, char lcSymbol);
	bool bIsEmpty(char lcSymbol);

	// Work out threat/opportunity score of line on board
	int cAssessLine(CBoard &lBoard, char lcSymbol, char lcLineIndex);

	// Work out total threat/opportunity score of a square
	int cAssessSquare(CBoard &lBoard, char lcSymbol, char lcSquare);

public:
	
	// Works out the next computer move
	char ucNextBestMove(CBoard &lBoard, char lcSymbol);

	C_AI()
	{
		// Nothing on construction
	}
	~C_AI()
	{
		// Nothing on destruction
	}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This is where the game functionality is
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class CGame
{
private:

	// Players taking part in the game
	CPlayer Players[NUM_PLAYERS];

	// Game's board interface
	CBoard Board;

	// Artificial intelligence
	C_AI AI;

	// Which player's turn it is
	char cCurrentPlayer;
	
	// Ask player to input their details
	bool bQueryPlayerType(char lcPlayer);
	bool bQueryPlayerSymbol(char lcPlayer);
	bool bQueryPlayers(void);
	
	// Ask where player wants to put next Symbol, process and switch to next player
	char cQueryPlayerInput(char lcPlayer);

	// Switch to next player's turn
	void vNextPlayer(void);

	// Switch to next player's turn
	char cGetPlayerBySymbol(char lcSymbol);

public:
	
	// Draws welcome screen
	void vDrawIntro(void);

	// Sets up player's settings
	bool bSetUpPlayers(void);
	
	// Displays the game result
	void vDrawGameResult(char lcOutcome);

	// Displays the game result
	void vDrawBoard(void);
	void vDrawBoardWithSquareNumbers(void);

	// Clears the board
	void vClearBoard(void);
	
	// Draws the end of game screen
	void vDrawShutdown(void);
	bool bWantAnotherGame(void);
	
	// Ask where player wants to put next Symbol, process and switch to next player
	bool bProcessPlayerTurn(void);

	// Check if there is a winner yet
	char cCheckForWinOrDraw(void);
	
	CGame()
	{
		// Set current player to zero on creation
		cCurrentPlayer = NULL;
	}
	~CGame()
	{
		// Nothing to do on destruction
	}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This is the program's entry point - uses CGame class instance directly
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

int main(void)
{
	// The main game object interface instance
	CGame Game;

	// Start random number generator
	srand(time(NULL));

	// Stores game result - draw, winning player
	char lcGameResult;
	
	// Display welcome screen
	Game.vDrawIntro();

	// Display board layout
	Game.vDrawBoardWithSquareNumbers();

	// Play games until player had enough
	do
	{
		// Set up player types and names
		Game.bSetUpPlayers();

		// Clear the game result
		lcGameResult = NULL;

		// Clear the game board
		Game.vClearBoard();
		
		// Keep playing until we have a winner, loser or draw
		while(!lcGameResult)
		{
			// Display the game board's state
			Game.vDrawBoard();

			// Process player turn
			Game.bProcessPlayerTurn();

			// Check if the game has been completed
			lcGameResult = Game.cCheckForWinOrDraw();
		}
		
		// Display the game board's state
		Game.vDrawBoard();

		// Output who won, lost or if there was a draw
		Game.vDrawGameResult(lcGameResult);
	}
	while(Game.bWantAnotherGame()); // Keep going in the above loop until player had enough

	// Gracefully say goodbye
	Game.vDrawShutdown();
	
	// Return control to the OS
	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Class functions for CPlayer
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CPlayer::vSetPlayerType(char lcNewPlayerType)
{
	// Set new player type
	cPlayerType = lcNewPlayerType;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

char CPlayer::cGetPlayerType(void)
{
	// Retrieve player type
	return cPlayerType;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CPlayer::vSetPlayerSymbol(char lcNewPlayerSymbol)
{
	// Set new player type
	cPlayerSymbol = lcNewPlayerSymbol;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

char CPlayer::cGetPlayerSymbol(void)
{
	// Retrieve player type
	return cPlayerSymbol;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CPlayer::bSetPlayerName(char *lcNewPlayerName)
{
	// Check string length
	if(strlen(lcNewPlayerName) > MAX_PLAYER_NAME)
	{
		// Too long - clear name and return failiure
		vClearPlayerName();
		return false;
	}

	// Set new player name
	strcpy(cPlayerName, lcNewPlayerName);

	// Return success
	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CPlayer::vClearPlayerName(void)
{
	// Blank out the player's name
	memset(cPlayerName, NULL, sizeof(cPlayerName));
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

char *CPlayer::cGetPlayerName(void)
{
	// Return a pointer to the player's name
	return cPlayerName;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Class functions for CBoard
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

char CBoard::cCheckMatch(char lcFirstSquare, char lcSecondSquare, char lcThirdSquare)
{
	// Validate each square
	if(!bIsValidSquare(lcFirstSquare))return false;
	if(!bIsValidSquare(lcSecondSquare))return false;
	if(!bIsValidSquare(lcThirdSquare))return false;

	// If there is a match, return it
	if(cBoard[lcFirstSquare] == cBoard[lcSecondSquare] && cBoard[lcSecondSquare] == cBoard[lcThirdSquare])
		return cBoard[lcFirstSquare];
	
	// No match
	return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

char CBoard::cArraySquare(char lcColumn, char lcRow)
{
	// Work out which square in the array we want to access
	return (lcRow * BOARD_WIDTH) + lcColumn;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CBoard::bIsValidSquare(char lcSquare)
{
	// Check the square for validity
	if(lcSquare >= BOARD_SIZE)return false; // Overflow?
	if(lcSquare < NULL)return false; // Underflow?
	
	// All clear
	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
void CBoard::vClearBoard(void)
{
	// Clear the board
	memset(cBoard, NULL, sizeof(cBoard));
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

char CBoard::cGetSquareSymbol(char lcSquare)
{
	// Check for errors in the input
	if(!bIsValidSquare(lcSquare))return false;

	// Return the square's contents
	return cBoard[lcSquare];
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CBoard::vSetSquareSymbol(char lcSquare, char lcSymbol)
{
	// Check for errors in the input
	if(!bIsValidSquare(lcSquare))return false;

	// Set the square's contents
	cBoard[lcSquare] = lcSymbol;

	// Affirmative
	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

char CBoard::cGetCompletedLine(void)
{
	// Used to cycle through the lines
	int liLinesIndex;

	// Holds result of line check
	char lcMatchSymbol;

	// Go through all the lines and see if any have matched Symbols
	for(liLinesIndex = NULL; liLinesIndex < MAX_LINES; liLinesIndex++)
	{
		// Check this line
		lcMatchSymbol = cCheckMatch(gcBoardLines[liLinesIndex][0], gcBoardLines[liLinesIndex][1], gcBoardLines[liLinesIndex][2]);
		
		// If there is a common Symbol on all squars of the line, return it
		if(lcMatchSymbol)
			return lcMatchSymbol;
	}

	// No matches
	return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CBoard::bIsFull(void)
{
	// Loops through all the squares
	int iSquareIndex;

	// Go through all the squares
	for(iSquareIndex = NULL; iSquareIndex < BOARD_SIZE; iSquareIndex++)
	{
		// If there is an empty square, the board is not full
		if(!cBoard[iSquareIndex])
			return false;
	}
	
	// The board is full
	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CBoard::bIsEmpty(void)
{
	// Loops through all the squares
	int iSquareIndex;

	// Go through all the squares
	for(iSquareIndex = NULL; iSquareIndex < BOARD_SIZE; iSquareIndex++)
	{
		// If there is a filled square, the board is full
		if(cBoard[iSquareIndex])
			return false;
	}
	
	// The board is empty
	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CBoard::vDrawBoardWithSquareNumbers(void)
{
	// Display board layout
	cout << endl;
	cout << "Board layout:" << endl;
	cout << endl;
	cout << ' ' << "1" << ' ' << '|' << ' ' << "2" << ' ' << '|' << ' ' << "3" << ' ' << endl;
	cout << "---|---|---" << endl;
	cout << ' ' << "4" << ' ' << '|' << ' ' << "5" << ' ' << '|' << ' ' << "6" << ' ' << endl;
	cout << "---|---|---" << endl;
	cout << ' ' << "7" << ' ' << '|' << ' ' << "8" << ' ' << '|' << ' ' << "9" << ' ' << endl;
	cout << "---|---|---"  << endl;
	cout << endl;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CBoard::vDrawBoard(void)
{
	// Clear the screen up
	system("cls");

	// Display board
	cout << ' ' << cBoard[0] << ' ' << '|' << ' ' << cBoard[1] << ' ' << '|' << ' ' << cBoard[2] << ' ' << endl;
	cout << "---|---|---" << endl;
	cout << ' ' << cBoard[3] << ' ' << '|' << ' ' << cBoard[4] << ' ' << '|' << ' ' << cBoard[5] << ' ' << endl;
	cout << "---|---|---" << endl;
	cout << ' ' << cBoard[6] << ' ' << '|' << ' ' << cBoard[7] << ' ' << '|' << ' ' << cBoard[8] << ' ' << endl;
	cout << "---|---|---"  << endl;
	cout << endl;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Class functions for C_AI
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool C_AI::bIsEnemy(char lcFriendly, char lcSymbol)
{
	// Friend or foe? empty ones are not foes, just not friends
	if(lcFriendly != lcSymbol && lcSymbol != NULL)
		return true;
	else
		return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool C_AI::bIsFriend(char lcFriendly, char lcSymbol)
{
	// Must be equal to be ours
	return (lcFriendly == lcSymbol);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool C_AI::bIsEmpty(char lcSymbol)
{
	// Must be NULL to be empty
	return (lcSymbol == NULL);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

int C_AI::cAssessLine(CBoard &lBoard, char lcSymbol, char lcLineIndex)
{
	// Holds cumulative line score
	char lcLine[BOARD_WIDTH];

	// Copy over the line matrix cell reference to a local array
	memcpy(lcLine, gcBoardLines[lcLineIndex], BOARD_WIDTH);

	// Put the actual values from the board into a single horizontal row of 3
	lcLine[0] = lBoard.cGetSquareSymbol(lcLine[0]);
	lcLine[1] = lBoard.cGetSquareSymbol(lcLine[1]);
	lcLine[2] = lBoard.cGetSquareSymbol(lcLine[2]);

	// Full, Full, Full
	if(!bIsEmpty(lcLine[0])            && !bIsEmpty(lcLine[1])           && !bIsEmpty(lcLine[2]))
	{
		return AI_LINESCORE_NO;
	}

	// Friend, Friend, Empty || Friend, Empty, Friend (mirroed included)
	if((bIsFriend(lcSymbol, lcLine[0]) && bIsFriend(lcSymbol, lcLine[1]) && bIsEmpty(lcLine[2]))
	|| (bIsEmpty(lcLine[0])            && bIsFriend(lcSymbol, lcLine[1]) && bIsFriend(lcSymbol, lcLine[2]))
	|| (bIsFriend(lcSymbol, lcLine[0]) && bIsEmpty(lcLine[1])            && bIsFriend(lcSymbol, lcLine[2])))
	{
		return AI_LINESCORE_WIN;
	}

	// Enemy, Enemy, Empty || Enemy, Empty, Enemy (mirroed included)
	if((bIsEnemy(lcSymbol, lcLine[0]) && bIsEnemy(lcSymbol, lcLine[1]) && bIsEmpty(lcLine[2]))
	|| (bIsEmpty(lcLine[0])           && bIsEnemy(lcSymbol, lcLine[1]) && bIsEnemy(lcSymbol, lcLine[2]))
	|| (bIsEnemy(lcSymbol, lcLine[0]) && bIsEmpty(lcLine[1])           && bIsEnemy(lcSymbol, lcLine[2])))
	{
		return AI_LINESCORE_PREVENT_LOSE;
	}

	// Friend, Empty, Empty || Empty, Friend, Empty (mirroed included)
	if((bIsFriend(lcSymbol, lcLine[0]) && bIsEmpty(lcLine[1])            && bIsEmpty(lcLine[2]))
	|| (bIsEmpty(lcLine[0])            && bIsEmpty(lcLine[1])            && bIsFriend(lcSymbol, lcLine[2]))
	|| (bIsEmpty(lcLine[0])            && bIsFriend(lcSymbol, lcLine[1]) && bIsEmpty(lcLine[2])))
	{
		return AI_LINESCORE_DEVELOP;
	}

	// Enemy, Empty, Empty || Empty, Enemy, Empty (mirroed included)
	if((bIsEnemy(lcSymbol, lcLine[0]) && bIsEmpty(lcLine[1])           && bIsEmpty(lcLine[2]))
	|| (bIsEmpty(lcLine[0])           && bIsEmpty(lcLine[1])           && bIsEnemy(lcSymbol, lcLine[2]))
	|| (bIsEmpty(lcLine[0])           && bIsEnemy(lcSymbol, lcLine[1]) && bIsEmpty(lcLine[2])))
	{
		return AI_LINESCORE_PREVENT_DEVELOP;
	}

	// Empty, Empty, Empty
	if(bIsEmpty(lcLine[0])            && bIsEmpty(lcLine[1])           && bIsEmpty(lcLine[2]))
	{
		return AI_LINESCORE_INTRODUCE;
	}

	// The only other solutions are:
	// Friend, Empty, Enemy || Friend, Enemy, Empty || Enemy, Friend, Empty
	return AI_LINESCORE_BATTLE;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

int C_AI::cAssessSquare(CBoard &lBoard, char lcSymbol, char lcSquare)
{
	// Local variables
	int liSquareScore = NULL;
	char lcLineIndex = NULL;
	char lcLinePartIndex = NULL;

	// Go through all the lines
	for(lcLineIndex = NULL; lcLineIndex < MAX_LINES; lcLineIndex++)
	{
		// Go through all the segments of the line
		for(lcLinePartIndex = NULL; lcLinePartIndex < BOARD_WIDTH; lcLinePartIndex++)
		{
			// If the line goes through this square
			if(gcBoardLines[lcLineIndex][lcLinePartIndex] == lcSquare)
			{
				// Cumulate the score of beign on that line
				liSquareScore += cAssessLine(lBoard, lcSymbol, lcLineIndex);

				// Skip to next line
				break;
			}
		}
	}

	// Return the score for this square as a potential destination
	return liSquareScore;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

char C_AI::ucNextBestMove(CBoard &lBoard, char lcSymbol)
{
	// Cycling variable for loops
	char lcSquareIndex = NULL;

	// Holds square we are planning to go to
	char lcSquare = NULL;
	
	// Best score and score of current square in assesment
	int liSquareScore = NULL, liNewSquareScore = NULL;

	// If the board is empty, simply pick a random starting point
	if(lBoard.bIsEmpty())
	{
		lcSquare = rand()%BOARD_SIZE;
		return lcSquare;
	}
	
	// Artificial intelligence processing - weigh each square up with respect to:
	// a) Number of enemy moves it will block
	// b) How close it will be to completing a line

	// Go through all the squares, assessing each one
	for(lcSquareIndex = NULL; lcSquareIndex < BOARD_SIZE; lcSquareIndex++)
	{
		if(!lBoard.cGetSquareSymbol(lcSquareIndex))
		{
			// Get score for this square
			liNewSquareScore = cAssessSquare(lBoard, lcSymbol, lcSquareIndex);

			// Compare to best score
			if(liNewSquareScore > liSquareScore)
			{
				// This is the new best square
				liSquareScore = liNewSquareScore;
				lcSquare = lcSquareIndex;
			}
		}
	}

	// Desparation - find random squares
	while(lBoard.cGetSquareSymbol(lcSquare))
	{
		// Try random square
		lcSquare = rand()%BOARD_SIZE;

		// Is the board full up - error?
		if(lBoard.bIsFull())
			return false;
	}

	// Return square to go this turn
	return lcSquare;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Class functions for CGame
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CGame::bQueryPlayerType(char lcPlayer)
{
	// Stores keystroke
	char lcInput = NULL;

	// Ask to enter H or C
	cout << "Player " << ((unsigned int)lcPlayer + 1) << " [H]uman or [C]omputer? " << endl;

	// Keep looping until we have valid input
	while(true)
	{
		// Get keystroke
		lcInput = getch();

		switch(lcInput)
		{
		case 'H':
		case 'h':
			// Selected human
			lcInput = PLAYER_HUMAN;
			Players[lcPlayer].vSetPlayerType(lcInput);
			cout << "Player " << ((unsigned int)lcPlayer + 1) << " is a human." << endl << endl;
			return true;
			break;
		case 'C':
		case 'c':
			// Selected computer
			lcInput = PLAYER_PC;
			Players[lcPlayer].vSetPlayerType(lcInput);
			cout << "Player " << ((unsigned int)lcPlayer + 1) << " is a computer." << endl << endl;
			return true;
			break;
		default:
			// Invalid input
			lcInput = NULL;
			// Prompt again
			cout << "Please press " << ((unsigned int)lcPlayer + 1) << " 'H' for human or 'C' for computer: " << endl;
			break;
		}
	}

	// Impossible
	return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CGame::bQueryPlayerSymbol(char lcPlayer)
{
	// Input not implemented
	return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CGame::bQueryPlayers(void)
{
	// For looping
	int liIndex;

	// Go through all the players
	for(liIndex = NULL; liIndex < NUM_PLAYERS; liIndex++)
	{
		// Ask to specify player type
		if(!bQueryPlayerType(liIndex))return false;
	}

	// Clear to next line
	cout << endl;

	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

char CGame::cQueryPlayerInput(char lcPlayer)
{
	// Stores keystroke
	char lcInput = NULL;

	// Ask to enter H or C
	cout << "Player " << ((unsigned int)cCurrentPlayer + 1) << ", choose a square: " << endl;

	// Keep looping until we have valid input
	while(true)
	{
		// Get keystroke in terms of numerical vlaue
		lcInput = getche() - 48;

		// Take one off for unit to zero conversion
		lcInput--;

		// Check the square selected square is valid
		if(!Board.bIsValidSquare(lcInput))
		{
			// Played his cards wrong, carry on looping
			cout << endl << "Invalid square, choose again:" << endl;
			lcInput = NULL;
			continue;
		}

		// Check the square selected square is not taken
		if(Board.cGetSquareSymbol(lcInput))
		{
			// Nadda, this square is already squatted
			cout << endl << "Square " << ((unsigned int)lcInput + 1) << " taken, choose again:" << endl;
			lcInput = NULL;
			continue;
		}

		// Move to next line
		cout << endl << endl;

		// Return the square player wants to play
		return lcInput;
	}

	// Inpossible
	return false;	
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CGame::vNextPlayer(void)
{
	// Increment player turn
	cCurrentPlayer++;

	// If cycled through all the players, start with the first one again
	if(cCurrentPlayer >= NUM_PLAYERS)
		cCurrentPlayer = NULL;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

char CGame::cGetPlayerBySymbol(char lcSymbol)
{
	// For looping
	int liIndex;

	// Go through all the players
	for(liIndex = NULL; liIndex < NUM_PLAYERS; liIndex++)
	{
		// Ask to specify player type
		if(Players[liIndex].cGetPlayerSymbol() == lcSymbol)
			return liIndex + 1; // Add one to avoid "fail" code - calling functions will need to subtract
	}

	// No such player
	return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CGame::vDrawIntro(void)
{
	// Display intro
	cout << "Welcome to Tic-Tac-Toe!" << endl << endl;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CGame::bSetUpPlayers(void)
{
	// Get player settings
	bQueryPlayers();

	// Set up other settings programmatically
	Players[0].vSetPlayerSymbol('X');
	Players[1].vSetPlayerSymbol('O');

	// Ok
	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	
void CGame::vDrawGameResult(char lcOutcome)
{
	// Holds player number of winner, if there is one
	char cWinner;

	// No result
	if(!lcOutcome)return;

	// How did the game end?
	if(lcOutcome != GAME_DRAW)
	{
		// Someone won, get their player number
		cWinner = cGetPlayerBySymbol(lcOutcome) - 1;
		
		// Congradulate them
		cout << "Player " << ((unsigned int)cWinner + 1) << " (" << Players[cWinner].cGetPlayerSymbol() << ") WON!" << endl;
	}
	else
	{
		// The game was drawn
		cout << "It's a DRAW!" << endl;
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CGame::vDrawBoard(void)
{
	// Call to draw the board
	Board.vDrawBoard();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CGame::vDrawBoardWithSquareNumbers(void)
{
	// Call to draw the board
	Board.vDrawBoardWithSquareNumbers();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CGame::vClearBoard(void)
{
	// Call to clear board
	Board.vClearBoard();

	// Reset current player
	cCurrentPlayer = NULL;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CGame::vDrawShutdown(void)
{
	// Display goodbye screen
	cout << endl << "Thanks for playing Tic-Tac-Toe, any key to close." << endl;

	// Pause for user to quit
	getch();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CGame::bWantAnotherGame(void)
{
	// Stores keystroke
	char cInput = NULL;

	// Ask to enter H or C
	cout << "Another game [Y/N]?" << endl;

	// Get keystroke
	cInput = getch();

	// What did the user enter?
	switch(cInput)
	{
	case 'Y':
	case 'y':
		// Yes
		return true;
		break;
	case 'N':
	case 'n':
	default:
		// Invalid input, assume they want to get out
		return false;
		break;
	}

	// Impossible
	return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CGame::bProcessPlayerTurn(void)
{
	// Square where the player intends to move
	unsigned char lcSquare;

	if(Players[cCurrentPlayer].cGetPlayerType() == PLAYER_HUMAN)
	{
		// Get player to enter square number
		lcSquare = cQueryPlayerInput(cCurrentPlayer);
	}
	else if(Players[cCurrentPlayer].cGetPlayerType() == PLAYER_PC)
	{
		// Get player to enter square number
		lcSquare = AI.ucNextBestMove(Board, Players[cCurrentPlayer].cGetPlayerSymbol());

		// Make impression of a player entering the number
		cout << "Player " << ((unsigned int)cCurrentPlayer + 1) << ", choose a square: " << endl;
		
		// Delay - pretend thinking time of computer
		Sleep(COMPUTER_DELAY_TIME);

		// Display computer's choice
		cout << ((int)lcSquare + 1) << endl << endl;
	}
	else return false;

	// Set the square on the board
	Board.vSetSquareSymbol(lcSquare, Players[cCurrentPlayer].cGetPlayerSymbol());

	// Next player's go
	vNextPlayer();

	// Ok
	return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

char CGame::cCheckForWinOrDraw(void)
{
	// Is there a winning symbol?
	char cWinningSymbol = Board.cGetCompletedLine();

	// Is the board filled up?
	bool bBoardFull = Board.bIsFull();

	// Did someone win?
	if(cWinningSymbol)
	{
		// Find out if there is such a player
		if(!cGetPlayerBySymbol(cWinningSymbol))
			return false;

		// Someone won
		return cWinningSymbol;
	}
	// If not, is it a draw?
	else if(bBoardFull)
	{
		// A draw
		return GAME_DRAW;
	}

	// No winner, no draw
	return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////